<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Photoslurp
 * @subpackage Photoslurp/admin
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 *
	 * @param    string $plugin_name The name of this plugin.
	 * @param    string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		$this->includes();

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
	}

	/**
	 * Admin includes
	 *
	 * @since    1.0.0
	 */
	public function includes() {
		/**
		 * Notification class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-photoslurp-admin-notifications.php';

		/**
		 * Meta box class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-photoslurp-admin-meta-boxes.php';

		new Photoslurp_Admin_Meta_Boxes();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/photoslurp-admin.css', array(), filemtime( plugin_dir_path( __FILE__ ) . 'css/photoslurp-admin.css' ), 'all' );
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/photoslurp-admin.js', array( 'jquery' ), filemtime( plugin_dir_path( __FILE__ ) . 'js/photoslurp-admin.js' ) . 'js/photoslurp-admin.js', false );
	}

	/**
	 * Register settings
	 *
	 * @since    1.0.0
	 */
	public function admin_init() {
		register_setting( 'photoslurp-settings', 'photoslurp_options' );

		add_settings_section(
			'photoslurp_section_products_csv',
			__( 'Products CSV settings', 'photoslurp' ),
			array( $this, 'photoslurp_section' ),
			'photoslurp-settings'
		);

		add_settings_field(
			'photoslurp_field_products_csv_identifier',
			__( 'Product identifier', 'photoslurp' ),
			array( $this, 'photoslurp_field' ),
			'photoslurp-settings',
			'photoslurp_section_products_csv',
			array(
				'id' => 'photoslurp_field_products_csv_identifier',
                'options' => array(
                    'sku' => __( 'SKU', 'photoslurp' ),
                    'product_id' => __( 'Product ID', 'photoslurp' )
                ),
                'description' => __( 'Unique identifier to use for the products CSV. When using the SKU, do not forget to enter a product SKU for parent products.', 'photoslurp' )
			)
		);

        add_settings_section(
            'photoslurp_section_widget',
            __( 'Widget settings', 'photoslurp' ),
            array( $this, 'photoslurp_section' ),
            'photoslurp-settings'
        );

        add_settings_field(
            'photoslurp_field_enable_lang_parameter',
            __( 'Override lang parameter', 'photoslurp' ),
            array( $this, 'photoslurp_field' ),
            'photoslurp-settings',
            'photoslurp_section_widget',
            array(
                'id' => 'photoslurp_field_enable_lang_parameter',
                'options' => array(
                    '0' => __( 'Disable', 'photoslurp' ),
                    '1' => __( 'Enable', 'photoslurp' )
                ),
                'description' => __( 'Enable to override the default lang parameter for each widget. If you are using the Product CSV of this plugin it is not advised to override the lang parameter. When enabled you can change the lang parameter for each widget under the \'Advanced\' tab.', 'photoslurp' )
            )
        );
	}

	/**
	 * Output section HTML
	 *
	 * @since    1.0.0
	 *
	 * @param    $args
	 */
	public function photoslurp_section( $args ) {

	}

	/**
	 * Output settings field HTML
	 *
	 * @since    1.0.0
	 *
	 * @param    $args
	 */
	public function photoslurp_field( $args ) {
		$options = get_option( 'photoslurp_options' );
		?>
        <select id="<?php echo esc_attr( $args['id'] ); ?>" name="photoslurp_options[<?php echo esc_attr( $args['id'] ); ?>]">
            <?php foreach( $args['options'] as $value => $label ): ?>
                <option value="<?php echo $value ?>" <?php echo isset( $options[ $args['id'] ] ) ? ( selected( $options[ $args['id'] ], $value, false ) ) : ( '' ); ?>>
                    <?php echo $label; ?>
                </option>
            <?php endforeach; ?>
        </select>
        <?php if( ! empty( $args['description'] ) ): ?>
            <p class="description">
                <?php echo $args['description'] ?>
            </p>
        <?php
        endif;
	}

	/**
	 * Add Photoslurp to the admin menu
	 *
	 * @since    1.0.0
	 */
	public function admin_menu() {
		/**
		 * Add the main menu item
		 */
		add_menu_page( __( 'Photoslurp', 'photoslurp' ), __( 'Photoslurp', 'photoslurp' ), 'edit_posts', 'photoslurp', null, plugins_url( 'images/photoslurp_menu_icon.png', __FILE__ ), '55.5' );

		/**
		 * Add a sub menu page for the product CSV
		 */
		add_submenu_page( 'photoslurp', __( 'Product CSV', 'photoslurp' ), __( 'Product CSV', 'photoslurp' ), 'edit_posts', 'photoslurp-product-csv', array(
			$this,
			'admin_product_csv'
		) );

		/**
		 * Add settings page
		 */
		add_submenu_page( 'photoslurp', __( 'Settings', 'photoslurp' ), __( 'Settings', 'photoslurp' ), 'manage_options', 'photoslurp-settings', array(
			$this,
			'admin_settings'
		) );
	}

	/**
	 * Product CSV admin page
	 *
	 * @since    1.0.0
	 */
	public function admin_product_csv() {
		$this->check_version();

		if ( isset( $_POST['regenerate_csv'] ) ) {
			if ( isset( $_POST['photoslurp_regenerate_csv'] ) && wp_verify_nonce( $_POST['photoslurp_regenerate_csv'], 'photoslurp_regenerate_csv_action' ) ) {
			    $language = 'all';
			    if (isset($_POST['language_code'])) {
			        $language = $_POST['language_code'];
                }
				$generator = new Photoslurp_Product_CSV();
				$result = $generator->generate($language);
				if ( $result === true ) {
					echo '<div class="notice updated"><p>' . __( 'Product CSV generated', 'photoslurp' ) . '</p></div>';
				} else if ( $result === 'no_products' ) {
					echo '<div class="notice error"><p>' . __( 'No suitable products found. Please make sure your products have a SKU when using the SKU as product identifier', 'photoslurp' ) . '</p></div>';
				} else {
					echo '<div class="notice error"><p>' . __( 'Product CSV generation failed. Please check the permissions of the uploads folder', 'photoslurp' ) . '</p></div>';
				}
			}
		}

		$url = Photoslurp_File_Helper::get_plugin_upload_url() . '/products.csv';
		$file_exists = Photoslurp_File_Helper::file_exists( 'products.csv' );

		echo '<div class="wrap">';

		echo '<h2>' . __( 'Photoslurp product CSV', 'photoslurp' ) . '</h2>';

		if ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON === true ) {
			Photoslurp_Admin_Notifications::warning( __( 'Warning: WordPress cron seems to be disabled. The product CSV will not be generated or updated.', 'photoslurp' ) );
		} else if ( $file_exists === false ) {
			Photoslurp_Admin_Notifications::warning( __( 'The Photoslurp product CSV has not been generated yet. Wait for it to be automatically generated (once every 24 hours) or use the button below to generate it manually.', 'photoslurp' ) );
		} else {
			Photoslurp_Admin_Notifications::info_box( __( 'The Photoslurp product CSV will be updated automatically with the details from your Woocommerce product catalog using the Wordpress cron service.', 'photoslurp' ) );
		}

		if ( $file_exists === true ) {
			echo '<h2>' . __( 'Product CSV', 'photoslurp' ) . '</h2>';
			echo '<p>' . sprintf( __( 'Product CSV URL: %s', 'photoslurp' ), '<a href="' . $url . '?v=' . rand( 0, 9999999 ) . '" target="_blank">' . $url . '</a>' ) . '</p>';
        }

		$label = __( 'Generate Product CSV', 'photoslurp' );
		$description = __( 'Manually generate the product CSV. Please note this may take a while depending of the number of products.', 'photoslurp' );
		if ( $file_exists === true ) {
			$label = __( 'Regenerate Product CSV', 'photoslurp' );
			$description = __( 'Manually regenerate the product CSV. Please note this may take a while depending of the number of products.', 'photoslurp' );
		}

		echo '<h2>' . $label . '</h2>';
		echo '<p>' . $description . '</p>';

		echo '<form name="regenerate_csv" method="post" action="">';
		wp_nonce_field( 'photoslurp_regenerate_csv_action', 'photoslurp_regenerate_csv' );
		echo '<input type="submit" class="button button-primary" name="regenerate_csv" value="' . $label . '" />';
        echo '</form>';

		if ( Photoslurp_WPML_Helper::wpml_active() ) {
		    $languages = Photoslurp_WPML_Helper::get_languages();
		    echo '<h2>' . __( 'Multilingual Product CSV files', 'photoslurp' ) . '</h2>';
            echo '<p>' . __( 'You can also generate a separate product CSV file per language.', 'photoslurp' ) . '</p>';

		    echo '<table class="widefat striped">';
		    echo '<thead><tr>';
		    echo '<th>' . __( 'Language', 'photoslurp' ) . '</th>';
            echo '<th>' . __( 'Product CSV URL', 'photoslurp' ) . '</th>';
            echo '<th></th>';
		    echo '</tr></thead>';

		    echo '<tbody>';
		    foreach ( $languages as $language_code => $language ) {
                $url = Photoslurp_File_Helper::get_plugin_upload_url() . '/products_'.$language_code.'.csv';
                $file_exists = Photoslurp_File_Helper::file_exists( 'products_'.$language_code.'.csv' );
                $label = __( 'Generate Product CSV', 'photoslurp' );
                if ( $file_exists === true ) {
                    $label = __( 'Regenerate Product CSV', 'photoslurp' );
                }

                echo '<tr>';
                echo '<td>' . $language_code . '</td>';
                echo '<td>'. ( $file_exists ? '<a href="' . $url . '?v=' . rand( 0, 9999999 ) . '">' . $url . '</a>' : '' ) .'</td>';
                echo '<td>';
                echo '<form name="regenerate_csv_'.$language_code.'" method="post" action="">';
                wp_nonce_field( 'photoslurp_regenerate_csv_action', 'photoslurp_regenerate_csv' );
                echo '<input type="submit" class="button button-primary" name="regenerate_csv" value="' . $label . '" />';
                echo '<input type="hidden" name="language_code" value="' . $language_code . '" />';
                echo '</form>';
                echo '</td>';
                echo '</tr>';
            }
            echo '</tbody>';

		    echo '</table>';
        }

		echo '</div>';
	}

	/**
	 * Add settings page
	 *
	 * @since    1.0.0
	 */
	public function admin_settings() {
	    $this->check_version();

		if ( isset( $_GET['settings-updated'] ) ) {
			add_settings_error( 'photoslurp_messages', 'photoslurp_message', __( 'Settings saved', 'photoslurp' ), 'updated' );
		}

		settings_errors( 'photoslurp_messages' );
		?>
        <div class="wrap">
            <h2><?php echo __( 'Photoslurp settings', 'photoslurp' ) ?></h2>
            <form action="options.php" method="post">
				<?php
				settings_fields( 'photoslurp-settings' );
				do_settings_sections( 'photoslurp-settings' );
				submit_button( __( 'Save settings', 'photoslurp' ) );
				?>
            </form>
        </div>
		<?php
	}

	/**
	 * Check the WordPress version
	 */
	public function check_version() {
		global $wp_version;

		if ($wp_version < PHOTOSLURP_REQUIRED_WORDPRESS_VERSION) {
			add_settings_error('photoslurp_messages', 'photoslurp_version_message', sprintf(__('Please update WordPress to version %s or higher to use the Photoslurp plugin. The version of this WordPress installation is %s.', 'photoslurp'), PHOTOSLURP_REQUIRED_WORDPRESS_VERSION, $wp_version));
        }
    }

}
