<?php

/**
 * Widget placement meta box
 *
 * Provides information on how to display the
 * Photoslurp Widget
 *
 * @package    Photoslurp
 * @subpackage Photoslurp/admin
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Meta_Box_Widget_Placement {

	/**
	 * Constructor
	 *
	 * Add meta box and hook save action
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ), 10 );
		add_action( 'save_post', array( $this, 'save' ), 10, 2 );
	}

	/**
	 * Add the actual meta box
	 *
	 * @since    1.0.0
	 */
	public function add_meta_box() {
		add_meta_box( 'photoslurp-widget-placement', __( 'How to display' ), array(
			$this,
			'output'
		), 'photoslurp_widget', 'side' );
	}

	/**
	 * Output widget settings meta box
	 *
	 * @since    1.0.0
	 *
	 * @param    WP_Post $post
	 */
	public function output( $post ) {
        // In case of WPML, check if this post is an translation
        $translation = Photoslurp_WPML_Helper::post_is_translation($post->ID, $post->post_type);

		// Set nonce
		wp_nonce_field( 'photoslurp_widget_placement_settings_save', 'photoslurp_widget_placement_settings_nonce' );

		echo '<p>' . __( 'There are 3 ways to display this Photoslurp Widget:', 'photoslurp' ) . '</p>';

		echo '<p><strong>1. ' . __( 'Short code', 'photoslurp' ) . '</strong>
		<br/>' . __( 'Add this short code in any post', 'photoslurp' ) . '</p>';
		echo '<p><code>[photoslurp id="' . $post->ID . '"]</code></p>';

		echo '<p><strong>2. ' . __( 'Widget', 'photoslurp' ) . '</strong>
		<br/>' . sprintf( __( 'Add the Photoslurp Widget under Appearance -> Widgets to any sidebar and select <strong>%s</strong>', 'photoslurp' ), $post->post_title ) . '</p>';

		echo '<p><strong>3. ' . __( 'Product and category pages', 'photoslurp' ) . '</strong>';
        echo '<br/>'.__( 'You can also show the widget dynamically on product or category pages. Please see the settings under the \'Configuration\' tab and select \'product\' or \'category\' under the \'Page type\' setting.', 'photoslurp' );
		echo '<br/><br/>'.__( 'Please note: Overriding WooCommerce templates may prevent this option from working.', 'photoslurp' ).'</p>';

		// Get post meta values of all fields and output form
		Photoslurp_Form_Builder::build_form( $this->get_form_fields_with_values( $post->ID ), '', $translation );
	}

	/**
	 * Get post meta value for all form fields and
	 * add them to the array
	 *
	 * @since   1.0.0
	 *
	 * @param   $post_id
	 *
	 * @return  array
	 */
	public function get_form_fields_with_values( $post_id ) {
		$fields = $this->get_form_fields();

		foreach ( $fields as $key => $field ) {
			if ( ! empty( $field['id'] ) ) {
				$value = get_post_meta( $post_id, $field['id'], true );
				// Allow 0 values
				if ( ! empty( $value ) || $value === '0' ) {
					$fields[ $key ]['default'] = maybe_unserialize( $value );
				}
			}
		}

		return $fields;
	}

	/**
	 * Get form fields
	 *
	 * @since    1.0.0
	 * @return   array
	 */
	public function get_form_fields() {

		$settings = Photoslurp_Settings_Widget_Placement::get();

		if ( ! is_array( $settings ) ) {
			return array();
		}

		return $settings;
	}

	/**
	 * Save widget settings meta box
	 *
	 * @since    1.0.0
	 *
	 * @param    $post_id
	 * @param    $post
	 */
	public function save( $post_id, $post ) {

		if ( ! Photoslurp_Admin_Meta_Boxes::save( $post_id, $post ) ) {
			return;
		}

		// Check the nonce
		if ( empty( $_POST['photoslurp_widget_placement_settings_nonce'] ) || ! wp_verify_nonce( $_POST['photoslurp_widget_placement_settings_nonce'], 'photoslurp_widget_placement_settings_save' ) ) {
			return;
		}

		// Save the fields
		foreach ( $this->get_form_fields() as $field ) {
			if ( ! empty( $field['id'] ) ) {
				if ( isset( $_POST[ $field['id'] ] ) ) {
					update_post_meta( $post_id, $field['id'], $_POST[ $field['id'] ] );
				}
			}
		}
	}

}
