<?php

/**
 * Widget settings meta box
 *
 * Add meta box to manage widget settings
 *
 * @package    Photoslurp
 * @subpackage Photoslurp/admin
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Meta_Box_Widget_Settings {

	/**
	 * Constructor
	 *
	 * Add meta box and hook save action
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ), 10 );
		add_action( 'save_post', array( $this, 'save' ), 10, 2 );
	}

	/**
	 * Add the actual meta box
	 *
	 * @since    1.0.0
	 */
	public function add_meta_box() {
		add_meta_box( 'photoslurp-widget-settings', __( 'Photoslurp widget settings' ), array(
			$this,
			'output'
		), 'photoslurp_widget', 'normal', 'high' );
	}

	/**
	 * Output widget settings meta box
	 *
	 * @since    1.0.0
	 *
	 * @param    WP_Post $post
	 */
	public function output( $post ) {
	    // Get sections
		$sections = $this->get_sections();
		// Get post meta values of all fields and output form
		$settings = $this->get_form_fields_with_values( $post->ID );
        // In case of WPML, check if this post is an translation
        $translation = Photoslurp_WPML_Helper::post_is_translation($post->ID, $post->post_type);

		// Set nonce
		wp_nonce_field( 'photoslurp_widget_settings_save', 'photoslurp_widget_settings_nonce' );

		// Show information
		Photoslurp_Admin_Notifications::info_box( __( 'Fields marked with an asterisk (*) are required. The widget will not work if any of these fields are left empty. Other fields are optional. When left empty the default value will be used.', 'photoslurp' ) );

        if ($translation === true) {
            Photoslurp_Admin_Notifications::warning( __( 'This is a translation of another Photoslurp Widget. Edit the source language to change the values of the grey fields.', 'photoslurp' ) );
        }
		?>
        <div class="photoslurp-meta-box-settings">
            <div class="photoslurp-tabs">
				<?php foreach ( $sections as $section ): ?>
                    <a class="photoslurp-tab" href="#" data-tab="<?php echo $section['id'] ?>"><?php echo $section['label'] ?></a>
				<?php endforeach; ?>
            </div>
            <div class="photoslurp-tab-contents">
                <?php foreach ( $sections as $section ): ?>
                    <div class="photoslurp-tab-content" data-tab="<?php echo $section['id'] ?>">
                        <?php
                        Photoslurp_Form_Builder::build_form( $settings, $section['id'], $translation ); ?>
                    </div>
                <?php endforeach; ?>
            </div>
        </div>
		<?php
	}

	/**
	 * Get post meta value for all form fields and
	 * add them to the array
	 *
	 * @since   1.0.0
	 *
	 * @param   $post_id
	 *
	 * @return  array
	 */
	public function get_form_fields_with_values( $post_id ) {
		$fields = $this->get_form_fields();

		foreach ( $fields as $key => $field ) {
			if ( ! empty( $field['id'] ) ) {
				$value = get_post_meta( $post_id, $field['id'], true );
				// Allow 0 values
				if ( ! empty( $value ) || $value === '0' ) {
					$fields[ $key ]['default'] = maybe_unserialize( $value );
				}
			}
		}

		return $fields;
	}

	/**
	 * Get form fields
	 *
	 * @since    1.0.0
	 * @return   array
	 */
	public function get_form_fields() {

		$settings = Photoslurp_Settings_Widget::get();

		if ( ! is_array( $settings ) ) {
			return array();
		}

		return $settings;
	}

	/**
	 * Get setting sections
	 *
	 * @since    1.0.0
	 * @return   array
	 */
	public function get_sections() {

		$sections = Photoslurp_Settings_Widget::get_sections();

		if ( ! is_array( $sections ) ) {
			return array();
		}

		return $sections;
	}

	/**
	 * Save widget settings meta box
	 *
	 * @since    1.0.0
	 *
	 * @param    $post_id
	 * @param    $post
	 */
	public function save( $post_id, $post ) {

		if ( ! Photoslurp_Admin_Meta_Boxes::save( $post_id, $post ) ) {
			return;
		}

		// Check the nonce
		if ( empty( $_POST['photoslurp_widget_settings_nonce'] ) || ! wp_verify_nonce( $_POST['photoslurp_widget_settings_nonce'], 'photoslurp_widget_settings_save' ) ) {
			return;
		}

		// Save the fields
		foreach ( $this->get_form_fields() as $field ) {
			if ( ! empty( $field['id'] ) ) {
				if ( isset( $_POST[ $field['id'] ] ) ) {
					update_post_meta( $post_id, $field['id'], maybe_serialize( $_POST[ $field['id'] ] ) );
				}
			}
		}
	}

}