<?php

/**
 * Form builder
 *
 * @since      1.0.0
 * @package    Photoslurp
 * @subpackage Photoslurp/includes
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Form_Builder {

	/**
	 * Build form
	 *
	 * @since      1.0.0
	 *
	 * @param      array $fields
	 * @param      string $section
     * @param      bool $translation
	 */
	public static function build_form( $fields, $section = '', $translation = false ) {
		echo '<div class="photoslurp-form">';

		foreach ( $fields as $field ) {
			if ( empty( $field['section'] ) || $field['section'] === $section ) {
			    if ( $translation === true && ( empty( $field['translate'] ) || $field['translate'] === false ) ) {
			        $field['disabled'] = true;
                }
                Photoslurp_Form_Builder::build_field( $field );
			}
		}

		echo '</div>';
	}

	/**
	 * Build form field
	 *
	 * @since      1.0.0
	 *
	 * @param      array $field
	 */
	public static function build_field( $field ) {
		if ( empty( $field['type'] ) || empty( $field['id'] ) ) {
			return;
		}

		if ( $field['type'] === 'title' ) {
            Photoslurp_Form_Builder::title( $field );

			return;
		}

		$label    = ! empty( $field['label'] ) ? $field['label'] : '';
		$required = ! empty( $field['required'] ) && $field['required'] === true ? 'photoslurp-required' : '';
		$class    = ! empty( $field['class'] ) ? $field['class'] : '';

		$data = '';
		if ( ! empty( $field['show_when'] ) && is_array( $field['show_when'] ) ) {
		    /*
		     * Some field should only be shown when a plugin setting has
		     * a certain value.
		     */
		    if ( isset( $field['show_when']['option'] ) ) {
                $options  = get_option( 'photoslurp_options' );
		        foreach ( $field['show_when']['option'] as $key => $value ) {
                    $option = ! empty( $options[ $key ] ) ? $options[ $key ] : null;
                    if ( $option !== $value ) {
                        return;
                    }
                }
            } else {
                /*
                 * Some field should only be shown when another field has a
                 * certain value. This happens with javascript.
                 */
                foreach ( $field[ 'show_when' ] as $key => $value ) {
                    $data .= ' data-show-when-field="' . $key . '" data-show-when-field-value="' . $value . '" ';
                }
            }
		}

        if ( ! empty( $field['auto_reset'] ) && $field['auto_reset'] === true ) {
            $data .= ' data-auto-reset="1" ';
        }

		echo '<p class="photoslurp-form-field ' . $class . ' photoslurp-' . sanitize_title( $field['type'] ) . ' ' . $required . '" ' . $data . '>';
		echo '<label for="' . $field['id'] . '">' . $label . '</label>';

		switch ( $field['type'] ) {
			case 'text':
                Photoslurp_Form_Builder::text( $field );
				break;
			case 'textarea':
                Photoslurp_Form_Builder::textarea( $field );
				break;
			case 'select':
                Photoslurp_Form_Builder::select( $field );
				break;
		}

		echo '</p>';
	}

	/**
	 * Build text input
	 *
	 * @since      1.0.0
	 *
	 * @param      array $field
	 */
	public static function text( $field ) {
		$default = isset( $field['default'] ) ? $field['default'] : '';
		$disabled =  !empty( $field['disabled'] ) && $field['disabled'] === true ? 'disabled' : '';

		echo '<input type="text" name="' . $field['id'] . '" id="' . $field['id'] . '" value="' . $default . '" '.$disabled.' />';
		if ( ! empty( $field['description'] ) ) {
			echo '<span class="photoslurp-description">' . $field['description'] . '</span>';
		}
	}

	/**
	 * Build text input
	 *
	 * @since      1.0.0
	 *
	 * @param      array $field
	 */
	public static function textarea( $field ) {
		$default = isset( $field['default'] ) ? $field['default'] : '';
        $disabled =  !empty( $field['disabled'] ) && $field['disabled'] === true ? 'disabled' : '';

		echo '<textarea name="' . $field['id'] . '" id="' . $field['id'] . '" rows="15" '.$disabled.'>' . $default . '</textarea>';
		if ( ! empty( $field['description'] ) ) {
			echo '<span class="photoslurp-description">' . $field['description'] . '</span>';
		}
	}


	/**
	 * Build select
	 *
	 * @since      1.0.0
	 *
	 * @param      array $field
	 */
	public static function select( $field ) {
		if ( empty( $field['options'] ) ) {
			return;
		}

		$default = isset( $field['default'] ) ? $field['default'] : '';
        $disabled =  !empty( $field['disabled'] ) && $field['disabled'] === true ? 'disabled' : '';
        $multiple =  !empty( $field['multiple'] ) && $field['multiple'] === true ? 'multiple' : '';

		echo '<select name="' . $field['id'] . ( $multiple === 'multiple' ? '[]' : '' ) . '" id="' . $field['id'] . '" '.$disabled.' '.$multiple.'>';
		foreach ( $field['options'] as $value => $text ) {
			echo '<option value="' . $value . '" ' . Photoslurp_Form_Builder::selected( $default, $value, $multiple ) . '>' . $text . '</option>';
		}
		echo '</select>';
		if ( ! empty( $field['description'] ) ) {
			echo '<span class="photoslurp-description">' . $field['description'] . '</span>';
		}
	}

    /**
     * Check if given option should be selected
     *
     * @since   1.4.0
     *
     * @param   $default
     * @param   $value
     * @param   $multiple
     *
     * @return  string
     */
	public static function selected( $default, $value, $multiple ) {
	    if ( $multiple === 'multiple' ) {
	        if ( in_array( $value, $default ) ) {
	            return 'selected';
            }

            return '';
        }

        return selected( $default, $value );
    }

	/**
	 * Build title
	 *
	 * @since      1.0.0
	 *
	 * @param      array $field
	 */
	public static function title( $field ) {
		echo '<h3 class="photoslurp-form-title">' . $field['label'] . '</h3>';
	}

}
