<?php

/**
 * Product CSV
 *
 * Generates a Photoslurp compatible CSV product feed
 *
 * @since      1.0.0
 * @package    Photoslurp
 * @subpackage Photoslurp/includes
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Product_CSV {

	/**
	 * Generate the CSV product feed
	 *
	 * @since    1.0.0
     *
     * @param    string $given_language
	 *
	 * @return   bool
	 */
	public function generate($given_language = 'all') {
        $language = get_locale();
        $storeId = get_current_blog_id();
        $restore_language = $language;

        if (Photoslurp_WPML_Helper::wpml_active()) {
            if ($given_language === 'all') {
                // Switch to default language
                Photoslurp_WPML_Helper::switch_language(Photoslurp_WPML_Helper::get_default_language());
            } else {
                // Switch to given language
                Photoslurp_WPML_Helper::switch_language($given_language);
                $language = $given_language;
            }
        }
		$products = $this->get_products();

		if ( empty( $products ) ) {
			return 'no_products';
		}

		$upload_path = Photoslurp_File_Helper::get_plugin_upload_path();
		$file_name   = $given_language === 'all' ? 'products.csv' : 'products_'.$given_language.'.csv';
		$output      = @fopen( trailingslashit( $upload_path ) . $file_name, 'w' );

		if (!$output) {
			return 'no_file';
		}

        $headers = array(
            'sku',
            'image_url'
        );

		if (Photoslurp_WPML_Helper::wpml_active() && $given_language === 'all') {
		    // Add headers for all languages
		    $languages = Photoslurp_WPML_Helper::get_languages();
		    $currencies = Photoslurp_WPML_Helper::get_currencies();

		    foreach ($languages as $language_code => $language) {
		        $uniqueCode = $language_code.'_'.$storeId;

                $headers = array_merge( $headers, array(
                    'title_' . $uniqueCode,
                    'description_' . $uniqueCode,
                    'url_' . $uniqueCode,
                    'in_stock_' . $uniqueCode,
                    'product_types_' . $uniqueCode
                ) );
            }

            foreach ($currencies as $currencyCode => $currency) {
                $headers = array_merge( $headers, array(
                    'price_' . $currencyCode
                ) );
            }
        } else {
            $uniqueCode = $language.'_'.$storeId;

            $headers = array_merge( $headers, array(
                'title_' . $uniqueCode,
                'description_' . $uniqueCode,
                'url_' . $uniqueCode,
                'price_' . $uniqueCode,
                'in_stock_' . $uniqueCode,
                'product_types_' . $uniqueCode
            ) );
        }

		/**
		 * CSV headers
		 */
		fputcsv( $output, $headers, '|' );

		foreach ( $products as $product ) {
            if (Photoslurp_WPML_Helper::wpml_active() && $given_language === 'all') {
                $languages = Photoslurp_WPML_Helper::get_languages();
                $currencies = Photoslurp_WPML_Helper::get_currencies();
                $languageProducts = Photoslurp_WPML_Helper::get_translated_posts(get_post($product['id']));

                $row = array(
                    $product['sku'],
                    $product['image_url']
                );

                // Other languages
                foreach ($languages as $language_code => $language) {

                    // Switch to language
                    Photoslurp_WPML_Helper::switch_language($language_code);

                    // Set product data for all languages
                    if (isset($languageProducts[$language_code])) {
                        $_product = wc_get_product($languageProducts[$language_code]->element_id);

                        $row[] = $_product->get_title();
                        $row[] = preg_replace( '/\s+/', ' ', trim( strip_tags( $_product->get_description() ) ) );
                        $row[] = $_product->get_permalink();
                        $row[] = $_product->is_in_stock() ? 'in stock' : 'out of stock';
                        $row[] = Photoslurp_Product_Helper::get_product_type_by_product( $_product );
                    } else {
                        $row[] = '';
                        $row[] = '';
                        $row[] = '';
                        $row[] = '';
                        $row[] = '';
                    }

                }

                // Set price for all currencies
                foreach ($currencies as $currencyCode => $currency) {
                    $row[] = Photoslurp_WPML_Helper::get_product_price_by_currency($product['price'], $currencyCode);
                }

                // Add row to CSV
                fputcsv( $output, $row, '|' );
            } else {

                // Add row to CSV
                fputcsv( $output, array(
                    $product['sku'],
                    $product['image_url'],
                    $product['title'],
                    $product['description'],
                    $product['url'],
                    $product['price'],
                    $product['in_stock'],
                    $product['product_types']
                ), '|' );
            }
		}

		fclose( $output );

        if (Photoslurp_WPML_Helper::wpml_active()) {
            // Switch back to previous current language
            Photoslurp_WPML_Helper::switch_language($restore_language);
        }

		return true;
	}

	/**
	 * Get WooCommerce products
	 *
	 * @since    1.0.0
	 *
	 * @return   array
	 */
	public function get_products() {
		$options    = get_option( 'photoslurp_options' );
		$identifier = ! empty( $options['photoslurp_field_products_csv_identifier'] ) ? $options['photoslurp_field_products_csv_identifier'] : 'sku';
		$products   = array();

		$query = new WP_Query( array(
			'post_type'      => array( 'product' ),
			'post_status'    => 'publish',
			'posts_per_page' => - 1
		) );

		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();

				$product = wc_get_product( get_the_ID() );

				if ( $product ) {

					if ( ( $identifier === 'sku' && $product->get_sku() != '' ) || ( $identifier === 'product_id' && $product->get_id() ) ) {
						$post = get_post( $product->get_id() );
						$products[] = array(
						    'id'           => $product->get_id(),
							'sku'          => $identifier === 'sku' ? $product->get_sku() : $product->get_id(),
							'title'        => $product->get_title(),
							'description'  => preg_replace( '/\s+/', ' ', trim( strip_tags( $post->post_content ) ) ),
							'url'          => $product->get_permalink(),
							'image_url'    => wp_get_attachment_image_url( $product->get_image_id(), 'large' ),
							'price'        => $product->get_sale_price() > 0 ? $product->get_sale_price() : $product->get_price(),
							'in_stock'     => $product->is_in_stock() ? 'in stock' : 'out of stock',
                            'product_types' => Photoslurp_Product_Helper::get_product_type_by_product( $product )
						);
					}

				}
			}
			wp_reset_postdata();
		}

		return $products;
	}

}
