<?php

/**
 * Product helper
 *
 * Provides functionality related to WooCommerce products
 *
 * @since      1.4.0
 * @package    Photoslurp
 * @subpackage Photoslurp/includes
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Product_Helper {

    /**
     * Get product type of multiple products
     *
     * @since   1.4.0
     *
     * @param   array $product_id_or_sku_array
     *
     * @return  array
     */
    public static function get_product_type_by_product_ids_or_skus( $product_id_or_sku_array ) {
        $options        = get_option( 'photoslurp_options' );
        $identifier     = ! empty( $options['photoslurp_field_products_csv_identifier'] ) ? $options['photoslurp_field_products_csv_identifier'] : 'sku';
        $product_types  = array();

        if ( is_array( $product_id_or_sku_array ) ) {
            foreach ( $product_id_or_sku_array as $product_id ) {
                // If the product identifier is set to sku, get the product id by sku
                if ( $identifier === 'sku' ) {
                    $product_id = wc_get_product_id_by_sku( $product_id );
                }

                $product = wc_get_product( $product_id );
                if ( $product ) {
                    $product_types = array_merge( $product_types, Photoslurp_Product_Helper::get_product_type_by_product( $product, 'array' ) );
                }
            }
        }

        return array_unique( $product_types );
    }

    /**
     * Get product types of given product
     *
     * @since   1.4.0
     *
     * @param   WC_Product $product
     * @param   string $return
     *
     * @return  array|string
     */
    public static function get_product_type_by_product( WC_Product $product, $return = 'string' ) {
        $product_types  = array();
        $category_ids   = $product->get_category_ids();

        // If this is a product variation, get the categories from the parent product
        if ( $product instanceof WC_Product_Variation ) {
            $parent = wc_get_product( $product->get_parent_id() );
            if ( $parent ) {
                $category_ids = $parent->get_category_ids();
            }
        }

        // Loop product categories
        foreach ( $category_ids as $category_id ) {
            $product_types[] = Photoslurp_Product_Helper::get_product_type_by_category_id( $category_id );
        }

        if ($return === 'string') {
            // Return as comma separated string
            return implode( ',', $product_types );
        } else {
            return $product_types;
        }
    }

    /**
     * Get product type by category IDs
     *
     * @since   1.4.0
     *
     * @param   $category_ids
     *
     * @return  array
     */
    public static function get_product_type_by_category_ids( $category_ids ) {
        $product_types  = array();

        if ( is_array( $category_ids ) ) {
            foreach ( $category_ids as $category_id ) {
                if ( !empty( $category_id ) ) {
                    $product_types[] = Photoslurp_Product_Helper::get_product_type_by_category_id( $category_id );
                }
            }
        }

        return $product_types;
    }

    /**
     * Get product type by category ID
     *
     * @since   1.4.0
     *
     * @param   $category_id
     *
     * @return  string
     */
    public static function get_product_type_by_category_id( $category_id ) {
        $hierarchy = array();
        // Get category parents
        $ancestors_ids = get_ancestors( $category_id, 'product_cat' );

        if ( ! empty( $ancestors_ids ) ) {
            // Reverse array order because categories are given from lowest to highest in the hierarchy
            $ancestors_ids = array_reverse( $ancestors_ids );
            // Loop category parents
            foreach ( $ancestors_ids as $ancestor_id ) {
                $category = get_term_by( 'id', $ancestor_id, 'product_cat' );
                if ( $category !== false ) {
                    // Push to hierarchy
                    $hierarchy[] = $category->name;
                }
            }
        }

        // Finally push original category to hierarchy
        $category = get_term_by( 'id', $category_id, 'product_cat' );
        if ( $category !== false ) {
            $hierarchy[] = $category->name;
        }

        // Add full category hierarchy as product type separated by >
        return implode( ' > ', $hierarchy );
    }

    /**
     * Get a list of all product categories to be used in a select
     *
     * @since   1.4.0
     *
     * @return  array
     */
    public static function get_categories_for_select() {
        $product_categories =   get_terms( 'product_cat' );
        $options =              array( '' => '' );

        /** @var WP_Term $product_category */
        foreach ( $product_categories as $product_category ) {
            $options[ $product_category->term_id ] = $product_category->name;
        }

        return $options;
    }

}