<?php

/**
 * WPML helper
 *
 * Provides functionality for WPML support
 *
 * @since      1.3.0
 * @package    Photoslurp
 * @subpackage Photoslurp/includes
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_WPML_Helper {

    /**
     * Returns true if WPML is active
     *
     * @since    1.3.0
     *
     * @return   bool
     */
    public static function wpml_active() {
        return function_exists('icl_get_languages');
    }


    /**
     * Get all languages
     *
     * @since    1.3.0
     *
     * @return   array|bool
     */
    public static function get_languages() {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            return icl_get_languages('skip_missing=0');
        }

        return false;
    }

    /**
     * Get current language
     *
     * @since    1.3.0
     *
     * @return   bool|string
     */
    public static function get_current_language() {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            global $sitepress;
            if (method_exists($sitepress, 'get_current_language')) {
                return $sitepress->get_current_language();
            }
        }
        return get_locale();
    }

    /**
     * Get price in given currency
     *
     * @since    1.3.0
     *
     * @return   string
     */
    public static function get_default_language() {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            global $sitepress;
            if (method_exists($sitepress, 'get_default_language')) {
                return $sitepress->get_default_language();
            }
        }
        return get_locale();
    }

    /**
     * Switch language
     *
     * @since    1.3.0
     *
     * @param    $language
     *
     * @return   mixed
     */
    public static function switch_language($language) {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            global $sitepress;
            if (method_exists($sitepress, 'switch_lang')) {
                return $sitepress->switch_lang($language);
            }
        }
        return false;
    }

    /**
     * Get translated posts of given post
     *
     * @since    1.3.0
     *
     * @param    $post
     *
     * @return   array
     */
    public static function get_translated_posts($post) {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            global $sitepress;
            $element_type = apply_filters('wpml_element_type', $post->post_type);
            $parent_language_info = apply_filters('wpml_element_language_details', null, array('element_id' => $post->ID, 'element_type' => $post->post_type));
            return $sitepress->get_element_translations($parent_language_info->trid, $element_type, false, true);
        }
        return [];
    }

    /**
     * Get list of active currencies
     *
     * @since    1.3.0
     *
     * @return   array
     */
    public static function get_currencies() {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            global $woocommerce_wpml;
            if (isset($woocommerce_wpml->multi_currency)) {
                if (isset($woocommerce_wpml->multi_currency)) {
                    $currencies = $woocommerce_wpml->multi_currency->get_currencies(true);
                    if (is_array($currencies) && !empty($currencies)) {
                        return $currencies;
                    }
                }
            }
        }
        // Fallback
        $currency = get_woocommerce_currency();
        return array($currency => $currency);
    }

    /**
     * Get price in given currency
     *
     * @since    1.3.0
     *
     * @param    $price
     * @param    $currency
     *
     * @return   float
     */
    public static function get_product_price_by_currency($price, $currency) {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            global $woocommerce_wpml;
            if (isset($woocommerce_wpml->multi_currency) && isset($woocommerce_wpml->multi_currency->prices)) {
                $price = $woocommerce_wpml->multi_currency->prices->convert_price_amount($price, $currency);
                if (floatval($price) > 0) {
                    return floatval($price);
                }
            }
        }
        return $price;
    }

    /**
     * Returns true if given post is a translation
     * of another post
     *
     * @since    1.3.0
     *
     * @param    $post_id
     * @param    $post_type
     *
     * @return   bool
     */
    public static function post_is_translation($post_id, $post_type) {
        if (Photoslurp_WPML_Helper::wpml_active()) {
            $args = array('element_id' => $post_id, 'element_type' => $post_type);
            $information = apply_filters('wpml_element_language_details', null, $args);
            if (!empty($information->source_language_code)) {
                return true;
            }
        }

        return false;
    }

}