<?php

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Photoslurp
 * @subpackage Photoslurp/includes
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp {

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'photoslurp';
		$this->version     = PHOTOSLURP_VERSION;

		$this->load_dependencies();
		$this->set_locale();
		$this->load_admin();
		$this->load_public();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-photoslurp-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-photoslurp-public.php';

		/**
		 * Post types
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-post-types.php';

		/**
		 * Settings
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-settings.php';

		/**
		 * Form builder
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-form-builder.php';

		/**
		 * File helper
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-file-helper.php';

        /**
         * Product helper
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-product-helper.php';

		/**
		 * WPML helper
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-wpml-helper.php';

        /**
         * Product CSV generator
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-photoslurp-product-csv.php';

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Photoslurp_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		new Photoslurp_i18n();

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_admin() {
		if ( ! is_admin() ) {
			return;
		}

		new Photoslurp_Admin( $this->get_plugin_name(), $this->get_version() );
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_public() {
		new Photoslurp_Public( $this->get_plugin_name(), $this->get_version() );
	}

}
