<?php

/**
 * Photoslurp widget
 *
 * Functionality to output a Photoslurp widget
 *
 * This is not a WordPress widget but the actual
 * Photoslurp widget showing photos
 *
 * @package    Photoslurp
 * @subpackage Photoslurp/public
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Public_Widget {

	/**
	 * The photoslurp_widget post holding the widget settings
	 *
	 * @var      WP_Post
	 */
	protected $post;

	/**
	 * If one or multiple products are given the widget
     * will only show photos connected to these products.
     * Can be unique product IDs or product SKUs
	 *
	 * @var      array
	 */
	protected $product;

    /**
     * If one or multiple categories are given the widget
     * will only show photos of these categories products.
     * Should be an array of unique category IDs.
     *
     * @var      array
     */
    protected $category;

	/**
	 * Photoslurp_Public_Short_Codes constructor.
	 *
	 * @since    1.0.0
	 *
	 * @param    int $post_id The photoslurp_widget post id
	 */
	public function __construct( $post_id ) {

		if ( empty( $post_id ) ) {
			return;
		}

		$post = get_post( $post_id );

		if ( $post ) {
			$this->post                = $post;
			$post                      = null;
		}

	}

	/**
	 * Set the product(s)
	 *
	 * @since    1.0.0
	 *
	 * @param    array $product
	 */
	public function set_product( $product ) {
		$this->product = $product;
	}

    /**
     * Set the categorie(s)
     *
     * @since    1.4.0
     *
     * @param    array $category
     */
    public function set_category( $category ) {
        $this->category = $category;
    }

	/**
	 * Get settings of current photoslurp_widget post
	 *
	 * @since    1.0.0
	 *
	 * @return   array
	 */
	protected function get_settings() {
		if ( ! $this->post ) {
			return array();
		}

		// Get settings fields
		$fields = Photoslurp_Settings_Widget::get();

		if ( empty( $fields ) ) {
			return array();
		}

		$settings = array();

		// Loop settings fields and get post meta value
		foreach ( $fields as $field ) {

			if ( ! empty( $field['id'] ) ) {

				// Get post meta value for this settings field
				$value = get_post_meta( $this->post->ID, $field['id'], true );

				if ( ! empty( $value ) || $value === '0' ) {
					// Set value and strip prefix
					$settings[ str_replace( 'photoslurp_', '', $field['id'] ) ] = maybe_unserialize( $value );
				}

			}

		}

		return $settings;
	}

	/**
	 * Get widget output
	 *
	 * @since    1.0.0
	 *
	 * @return   string
	 */
	public function get_output() {
		$settings = $this->get_settings();

		if ( ! $this->has_required_settings( $settings ) ) {
			return '';
		}

		$widget_id = $settings['widget_id'];
		$optional_settings = $this->get_optional_settings( $settings );
        $custom_css = ! empty( $settings['custom_css'] ) ? $settings['custom_css'] : '';

		ob_start();

		if( ! empty( $custom_css ) ): ?>
            <style type="text/css" data-photoslurp-css-id="<?php echo $widget_id ?>">
                <?php echo $custom_css; ?>
            </style>
        <?php endif; ?>

        <!-- Start Photoslurp Additional Parameters Code -->
        <script>
            window.photoSlurpWidgetSettings = window.photoSlurpWidgetSettings || {};
            photoSlurpWidgetSettings["<?php echo $widget_id ?>"] = <?php echo json_encode($optional_settings) ?>;
        </script>
        <!-- End Photoslurp Additional Parameters Code -->

        <!-- Start Photoslurp Embed Code -->
        <div class="ps-container">
            <ps-widget data-config="<?php echo $widget_id ?>"></ps-widget>
            <script src="https://static.photoslurp.com/widget/v3/loader.js" async></script>
        </div>
        <!-- End Photoslurp Embed Code -->
		<?php

		return ob_get_clean();
	}

	/**
	 * Returns an array of optional settings only
	 * if they are given a value and map them to
	 * the Photoslurp Widget parameter names
	 *
	 * This array will be converted to JSON
	 * when used in the Photoslurp Widget
	 *
	 * @since    1.0.0
	 *
	 * @param    $settings
	 *
	 * @return   array
	 */
	protected function get_optional_settings( $settings ) {
        $options = get_option( 'photoslurp_options' );
        $enable_lang_parameter = isset( $options['photoslurp_field_enable_lang_parameter'] ) ? $options['photoslurp_field_enable_lang_parameter'] : null;
        $optional_settings = array();

		// General settings

		if ( ! empty( $settings['page_type'] ) ) {

			// Get product type if automatic product types is set to 'Yes'
            if ( in_array( $settings['page_type'], array( 'product', 'category' ) ) ) {

                if ( ! empty( $settings['automatic_product_types'] ) && $settings['automatic_product_types'] === '1' ) {
                    if ( ! empty( $this->product ) ) {
                        $optional_settings['productType'] = Photoslurp_Product_Helper::get_product_type_by_product_ids_or_skus( $this->product );
                    } else if ( ! empty( $this->category ) ) {
                        $optional_settings['productType'] = Photoslurp_Product_Helper::get_product_type_by_category_ids( $this->category );
                    }
                }

            // Get product type if there are given product categories
            } else if ( $settings['page_type'] === 'lookbook' ) {

                if ( ! empty( $settings['custom_product_types'] ) && is_array( $settings['custom_product_types'] ) ) {
                    $this->set_category( $settings['custom_product_types'] );
                    $optional_settings['productType'] = Photoslurp_Product_Helper::get_product_type_by_category_ids( $this->category );
                }

            }
		}

		/**
		 * If a product is set, add it to the optional settings to only
		 * show photos of this product
		 */
		if ( ! empty( $this->product ) ) {
			$optional_settings['productId'] = $this->product;
		}

        if ( ! empty( $settings['language'] ) && $enable_lang_parameter === '1' ) {
            $optional_settings['lang'] = $settings['language'];
        } else {
            if ( Photoslurp_WPML_Helper::wpml_active() ) {
                $language = Photoslurp_WPML_Helper::get_current_language();
            } else {
                $language = get_locale();
            }
            $optional_settings['lang'] = $language.'_'.get_current_blog_id();
        }

        return $optional_settings;
	}

	/**
	 * Check if required settings are given
	 *
	 * @param    $settings
	 *
	 * @since    1.0.0
	 *
	 * @return   bool
	 */
	protected function has_required_settings( $settings ) {

		if ( empty( $settings['user_name'] ) ||
		     empty( $settings['widget_id'] )
		) {
			return false;
		}

		return true;
	}

}
