<?php

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Photoslurp
 * @subpackage Photoslurp/public
 * @author     Tristan Hartman <tristan@hartmanmedia.nl>
 */
class Photoslurp_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 *
	 * @param    string $plugin_name The name of the plugin.
	 * @param    string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		add_action( 'wp', array( $this, 'display_widget_on_product_page' ) );
        add_action( 'wp', array( $this, 'display_widget_on_category_page' ) );

		$this->includes();
	}

	/**
	 * Public includes
	 *
	 * @since    1.0.0
	 */
	public function includes() {
		/**
		 * Photoslurp widget class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-photoslurp-public-widget.php';

		/**
		 * Short codes class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-photoslurp-public-short-codes.php';

		new Photoslurp_Public_Short_Codes();

		/**
		 * Widgets class
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-photoslurp-public-widgets.php';

		new Photoslurp_Public_Widgets();

		/**
		 * WooCommerce actions
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-photoslurp-public-woo-commerce-actions.php';

		new Photoslurp_Public_Woo_Commerce_Actions();

		/**
		 * Cron
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-photoslurp-public-cron.php';

        new Photoslurp_Public_Cron();
        
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/photoslurp-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/photoslurp-public.js', array( 'jquery' ), $this->version, false );

	}

    /**
     * Add Photoslurp widget to the single product page
     *
     * @since    1.3.1
     */
	public function display_widget_on_product_page() {
        
        if(is_product()){
            $query = new WP_Query( array(
                'post_type'      => 'photoslurp_widget',
                'posts_per_page' => - 1,
                'meta_key'       => 'photoslurp_show_on_product_page',
                'meta_value'     => '1'
            ));
            

            if ( $query->have_posts() ) {
                while ( $query->have_posts() ) {
                    $query->the_post();

                    // Get the hook name for this widget
                    $hook = get_post_meta( get_the_ID(), 'photoslurp_product_page_hook', true );

                    // Check if there is a custom hook name for this widget
                    if ( empty( $hook ) ) {
                        $hook = 'woocommerce_after_single_product';
                    } else if ( $hook === 'custom' ) {
                        $hook = get_post_meta( get_the_ID(), 'photoslurp_product_page_hook_custom', true );
                    }

                    // Hook should not be empty
                    if ( empty( $hook ) ) {
                        return;
                    }

                    $widget_id = get_the_ID();

                    add_action( $hook, function() use ($widget_id) { 

                        /** @var WC_Product $product */
                        global $product;

                        $options    = get_option( 'photoslurp_options' );
                        $identifier = ! empty( $options['photoslurp_field_products_csv_identifier'] ) ? $options['photoslurp_field_products_csv_identifier'] : 'sku';

                        $widget = new Photoslurp_Public_Widget( $widget_id );
                        $widget->set_product( array( ( $identifier === 'sku' ? $product->get_sku() : $product->get_id() ) ) );
                        echo $widget->get_output();

                    });
                }
                wp_reset_postdata();
            }
        }
    }

    /**
     * Add Photoslurp widget to the single product page
     *
     * @since    1.3.1
     */
    public function display_widget_on_category_page() {
        
        if(is_product_category()){
            $query = new WP_Query( array(
                'post_type'      => 'photoslurp_widget',
                'posts_per_page' => - 1,
                'meta_query'     => array(
                    array(
                        'key'     => 'photoslurp_show_on_category_page',
                        'value'   => array( '1', '2' ),
                        'compare' => 'IN',
                    )
                )
            ) );

            if ( $query->have_posts() ) {
                while ( $query->have_posts() ) {
                    $query->the_post();

                    // Get the hook name for this widget
                    $hook = get_post_meta( get_the_ID(), 'photoslurp_category_page_hook', true );

                    // Check if there is a custom hook name for this widget
                    if ( empty( $hook ) ) {
                        $hook = 'woocommerce_archive_description';
                    } else if ( $hook === 'custom' ) {
                        $hook = get_post_meta( get_the_ID(), 'photoslurp_category_page_hook_custom', true );
                    }

                    // Hook should not be empty
                    if ( empty( $hook ) ) {
                        return;
                    }

                    // Check if this widget should only be shown for selected categories (photoslurp_show_on_category_page = 2)
                    $show_on = get_post_meta( get_the_ID(), 'photoslurp_show_on_category_page', true );
                    $categories_to_show_on = maybe_unserialize(get_post_meta( get_the_ID(), 'photoslurp_category_pages_to_show_on', true ));
                    $categories_to_show_on = intval($show_on) === 2 && is_array($categories_to_show_on) ? $categories_to_show_on : array();

                    $widget_id = get_the_ID();

                    add_action( $hook, function() use ($widget_id, $categories_to_show_on) {

                        $term = get_queried_object();
                        if ( $term instanceof WP_Term ) {

                            if ( ! empty( $categories_to_show_on ) && ! in_array( $term->term_id, $categories_to_show_on ) ) {
                                return;
                            }

                            $widget = new Photoslurp_Public_Widget( $widget_id );
                            $widget->set_category( array( $term->term_id ) );
                            echo $widget->get_output();
                        }

                    });
                }
                wp_reset_postdata();
            }
        }
    }

}
